@@ -1,7 +1,9 @@
+from lambdatool.lambda_creator import LambdaCreator #noqa
+from lambdatool.lambda_deployer import LambdaDeployer #noqa
 from datetime import datetime
 
 __title__ = 'lambdatool'
-__version__ = '0.3.2'
+__version__ = '0.8.6'
 __author__ = 'Mr. Chuck Muckamuck'
 __author_email__ = 'chuck.muckamuck@gmail.com'
 __license__ = '''MIT License
