@@ -4,12 +4,13 @@ The command line interface to stackility.
 Major help from: https://www.youtube.com/watch?v=kNke39OZ2k0
 """
 import lambdatool
-from lambda_creator import LambdaCreator
-from lambda_deployer import LambdaDeployer
+from lambdatool.lambda_creator import LambdaCreator
+from lambdatool.lambda_deployer import LambdaDeployer
 import click
 import boto3
 import logging
 import sys
+import os
 import json
 
 default_stage = 'dev'
@@ -24,7 +25,7 @@ lambdatool deploy. The lambda has been started in main.py.
 
 
 @click.group()
-@click.version_option(version='0.3.2')
+@click.version_option(version='0.8.6')
 def cli():
     pass
 
@@ -59,6 +60,8 @@ def new(directory, name, service, profile, region):
     else:
         command_line['region'] = None
 
+    command_line['service'] = service
+
     if start_new_lambda(command_line):
         sys.exit(0)
     else:
@@ -102,6 +105,20 @@ def deploy(directory, stage, profile, region):
         sys.exit(1)
 
 
+@cli.command()
+@click.option('-s', '--stage', help='environment/stage of interest', required=True)
+def print_env(stage):
+    config_file = f'config/{stage}/function.properties'
+    if os.path.isfile(config_file):
+        with open(config_file, 'r') as f:
+            tmp = f.readline()
+            while tmp:
+                food = tmp.strip()
+                print(f'export {food}')
+                tmp = f.readline()
+
+
+
 def start_new_lambda(command_line):
     try:
         tool = LambdaCreator(command_line)
