@@ -1,3 +1,11 @@
+'''
+Facilitate the packaging and deployment of the function
+'''
+# pylint: disable=invalid-name
+# pylint: disable=broad-except
+# pylint: disable=line-too-long
+# pylint: disable=bare-except
+
 import logging
 import traceback
 import subprocess
@@ -9,11 +17,12 @@ import json
 import boto3
 import shutil
 import zipfile
-import utility #noqa
-import ConfigParser
-from stack_tool import StackTool
-from template_creator import TemplateCreator
+
+from configparser import ConfigParser
+from lambdatool.stack_tool import StackTool
+from lambdatool.template_creator import TemplateCreator
 from stackility import CloudStackUtility
+from lambdatool.pip_util import pipmain
 
 try:
     import zlib #noqa
@@ -28,10 +37,18 @@ logging.basicConfig(level=logging.INFO,
 logging.getLogger().setLevel(logging.INFO)
 
 
-PYTHON = 'python2.7'
+LAMBDATOOL_VERSION = '0.8.6'
+LAMBDATOOL_DESCRIPTOR = '.lambdatool'
+DEFAULT_DESCRIPTION = 'Fantastic Lambda Function'
 DEFAULT_MODULE_FILE = 'main.py'
 IGNORED_STUFF = ('config', '.git')
-PIP_COMMAND = 'pip install -q -Ur requirements.txt -t .'
+PIP_ARGS = [
+    'install',
+    '-Ur',
+    'requirements.txt',
+    '-t',
+    '.'
+]
 
 ZIP_MODES = {
     zipfile.ZIP_DEFLATED: 'deflated',
@@ -51,12 +68,16 @@ class LambdaDeployer:
     _profile = None
     _region = None
     _lambda_name = None
+    _create_log_group = False
+    _description = None
     _hash = None
     _tag_file = None
     _stack_properties = {}
     _template_directory = None
     _s3_client = None
+    _apig_client = None
     _ssm_client = None
+    _python = None
 
     def __init__(self, config_block):
         """
@@ -73,6 +94,7 @@ class LambdaDeployer:
         Raises:
             SystemError - if everything isn't just right
         """
+        self._stack_name = None
         if config_block:
             if not os.path.isdir(config_block['work_directory']):
                 logging.error('{} does not exist'.format(config_block['work_directory']))
@@ -89,6 +111,25 @@ class LambdaDeployer:
             logging.error('config block was garbage')
             raise SystemError
 
+        v = sys.version_info
+        if v.major == 2:
+            self._python = 'python2.7'
+        elif v.major == 3:
+            if v.minor == 6:
+                self._python = 'python3.6'
+            elif v.minor == 7:
+                self._python = 'python3.7'
+            elif v.minor == 8:
+                self._python = 'python3.8'
+            else:
+                logging.error('python %s.%s detected', v.major, v.minor)
+                logging.error('only python3.6, python3.7 and python3.8 are available')
+                raise SystemError
+        else:
+            logging.error('strange python version')
+            raise SystemError
+        logging.info('{} runtime selected'.format(self._python))
+
     def deploy_lambda(self):
         """
         Deploy an existing lambda to the indicated by creating CF template...
@@ -101,7 +142,10 @@ class LambdaDeployer:
             False if the lambda is not deployed for some odd reason
         """
         try:
-            self._lambda_name = self.find_lambda_name()
+            if not self.read_descriptor():
+                logging.error('problem reading ".lambdatool" descriptor, exiting')
+                return False
+
             if not self.verify_lambda_directory():
                 logging.error('module file {} not found, exiting'.format(DEFAULT_MODULE_FILE))
                 return False
@@ -173,12 +217,47 @@ class LambdaDeployer:
                 logging.info('create_stack() failed')
                 return False
 
+            if self.redeploy_api():
+                logging.info('redeploy_api() successful')
+            else:
+                logging.info('redeploy_api() failed')
+                return False
+
             return True
         except Exception as x:
             logging.error('Exception caught in deploy_lambda(): {}'.format(x))
             traceback.print_exc(file=sys.stdout)
             return False
 
+    def redeploy_api(self):
+        try:
+            rest_api_id = None
+            deployment_id = None
+
+            response = self._cf_client.describe_stack_resources(
+                StackName=self._stack_name
+            )
+
+            for resource in response['StackResources']:
+                if resource['ResourceType'] == 'AWS::ApiGateway::RestApi':
+                    rest_api_id = resource['PhysicalResourceId']
+                elif resource['ResourceType'] == 'AWS::ApiGateway::Deployment':
+                    deployment_id = resource['PhysicalResourceId']
+
+            if rest_api_id and deployment_id:
+                logging.debug('redeploying the API')
+                r = self._apig_client.create_deployment(
+                    restApiId=rest_api_id,
+                    stageName=self._stage
+                )
+                logging.debug(r)
+
+            return True
+        except Exception as wtf:
+            logging.error(wtf, exc_info=True)
+
+        return False
+
     def create_stack(self):
         try:
             ini_data = {}
@@ -186,7 +265,7 @@ class LambdaDeployer:
             ini_data['tags'] = {}
             ini_data['parameters'] = {}
 
-            stack_name = '{}-lambda-{}'.format(
+            self._stack_name = '{}-lambda-{}'.format(
                 self._stage,
                 self._lambda_name
             )
@@ -194,7 +273,7 @@ class LambdaDeployer:
             tmp_env = self._ini_data[self._stage]
             ini_data['environment']['template'] = '{}/template.yaml'.format(self._work_directory)
             ini_data['environment']['bucket'] = tmp_env['bucket']
-            ini_data['environment']['stack_name'] = stack_name
+            ini_data['environment']['stack_name'] = self._stack_name
             ini_data['codeVersion'] = self._hash
             if self._region:
                 ini_data['environment']['region'] = self._region
@@ -212,7 +291,7 @@ class LambdaDeployer:
                 if stack_driver.poll_stack():
                     logging.info('stack create/update was finished successfully.')
                     st = StackTool(
-                        stack_name,
+                        self._stack_name,
                         self._stage,
                         self._profile,
                         self._region,
@@ -234,20 +313,28 @@ class LambdaDeployer:
     def create_template_file(self):
         try:
             function_properties = '{}/config/{}/function.properties'.format(
-                    self._work_directory,
-                    self._stage
+                self._work_directory,
+                self._stage
             )
             output_file = '{}/template.yaml'.format(self._work_directory)
             template_file = '{}/template_template'.format(self._template_directory)
             templateCreator = TemplateCreator(self._ssm_client)
+
+            description = self._ini_data.get(self._stage, {}).get(
+                'description',
+                DEFAULT_DESCRIPTION
+            )
+
             template_created = templateCreator.create_template(
                 function_properties=function_properties,
+                description=description,
                 stack_properties=self._stack_properties,
                 output_file=output_file,
                 template_file=template_file,
                 region=self._region,
                 stage_name=self._stage,
                 short_name=self._lambda_name,
+                create_log_group=self._create_log_group,
                 account=boto3.client('sts').get_caller_identity()['Account']
             )
 
@@ -267,20 +354,33 @@ class LambdaDeployer:
             subnets = self._ini_data.get(self._stage, {}).get('subnets', None)
             sns_topic_arn = self._ini_data.get(self._stage, {}).get('snstopicarn', None)
             trusted_service = self._ini_data.get(self._stage, {}).get('trustedservice', None)
+            trusted_account = self._ini_data.get(self._stage, {}).get('trustedaccount', None)
+            reserved_concurrency = self._ini_data.get(self._stage, {}).get('reservedconcurrency', None)
             lambda_schedule_expression = self._ini_data.get(self._stage, {}).get('scheduleexpression', None)
             service = self._ini_data.get(self._stage, {}).get('service', None)
+            export_name = self._ini_data.get(self._stage, {}).get('export_name', None)
+            retention_days = self._ini_data.get(self._stage, {}).get('retention_days', '30')
+            whitelist = self._ini_data.get(self._stage, {}).get('whitelist', None)
 
             wrk = {}
             wrk['s3Bucket'] = bucket
             wrk['s3Key'] = self._package_key
             wrk['functionName'] = '{}-{}'.format(self._lambda_name, self._stage)
+            wrk['logGroupName'] = '/aws/lambda/{}-{}'.format(self._lambda_name, self._stage)
             wrk['handler'] = 'main.lambda_handler'
-            wrk['runTime'] = PYTHON
+            wrk['runTime'] = self._python
             wrk['memorySize'] = memory_size
             wrk['role'] = role
             wrk['timeOut'] = timeout
             wrk['securityGroupIds'] = security_group
             wrk['subnetIds'] = subnets
+            wrk['retentionDays'] = retention_days
+
+            if whitelist:
+                wrk['whitelist'] = whitelist
+
+            if export_name:
+                wrk['export_name'] = export_name
 
             if sns_topic_arn:
                 logging.info('subscribing lambda to SNS topic: {}'.format(sns_topic_arn))
@@ -290,6 +390,14 @@ class LambdaDeployer:
                 logging.info('the lambda will trust: {}'.format(trusted_service))
                 wrk['trustedService'] = trusted_service
 
+            if trusted_account:
+                logging.info('the lambda will trust: {}'.format(trusted_account))
+                wrk['trustedAccount'] = trusted_account
+
+            if reserved_concurrency:
+                logging.info('the lambda  will reserve {} invocations'.format(reserved_concurrency))
+                wrk['reservedConcurrency'] = reserved_concurrency
+
             if lambda_schedule_expression:
                 logging.info('the lambda will be scheduled by: {}'.format(lambda_schedule_expression))
                 wrk['scheduleExpression'] = lambda_schedule_expression
@@ -318,10 +426,9 @@ class LambdaDeployer:
                 outfile.write('APPLICATION={} lambda function\n'.format(self._lambda_name))
                 outfile.write('ENVIRONMENT={}\n'.format(self._stage))
                 outfile.write('STACK_NAME=lambda-{}-{}\n'.format(
-                        self._lambda_name,
-                        self._stage
-                    )
-                )
+                    self._lambda_name,
+                    self._stage
+                ))
                 outfile.write('VERSION={}\n'.format(self._hash))
             return True
         except Exception as x:
@@ -362,9 +469,14 @@ class LambdaDeployer:
             return False
 
     def install_requirements(self):
-        command = PIP_COMMAND.split()
-        exit_status, stdout_text, stderr_text = self.execute_command(command)
-        return exit_status == 0
+        try:
+            pipmain(PIP_ARGS)
+        except Exception as wtf:
+            logging.error('Exception caught in upload_package(): {}'.format(wtf))
+            traceback.print_exc(file=sys.stdout)
+            return False
+
+        return True
 
     def copy_stuff(self):
         try:
@@ -406,25 +518,26 @@ class LambdaDeployer:
     def verify_lambda_directory(self):
         return os.path.isfile(DEFAULT_MODULE_FILE)
 
-    def find_lambda_name(self):
-        lambdatool = '.lambdatool'
-        lambda_name = None
+    def read_descriptor(self):
         try:
-            with open(lambdatool, 'r') as j:
+            with open(LAMBDATOOL_DESCRIPTOR, 'r') as j:
                 stuff = json.load(j)
-                lambda_name = stuff['name']
+                self._lambda_name = stuff.get('name', None)
+                self._create_log_group = stuff.get('createLogGroup', False)
 
-            if not lambda_name:
+            if not self._lambda_name:
                 cwd = os.getcwd()
                 dirs = cwd.split('/')
-                lambda_name = dirs[-1]
+                self._lambda_name = dirs[-1]
 
-            logging.info('lambda_name: {}'.format(lambda_name))
+            logging.info('lambda_name: {}'.format(self._lambda_name))
+            logging.info('create_log_group: {}'.format(self._create_log_group))
+            return True
         except Exception as x:
-            logging.error('Exception caught in deploy_lambda(): {}'.format(x))
+            logging.error('Exception caught in read_descriptor(): {}'.format(x))
             traceback.print_exc(file=sys.stdout)
 
-        return lambda_name
+        return False
 
     def execute_command(self, command):
         stdout_buf = str()
@@ -443,7 +556,7 @@ class LambdaDeployer:
 
             return p.returncode, stdout_buf, stderr_buf
         except subprocess.CalledProcessError as x:
-            logging.error('Exception caught in create_lambda(): {}'.format(x))
+            logging.error('Exception caught in execute_command(): {}'.format(x))
             traceback.print_exc(file=sys.stdout)
             return x.returncode, None, None
 
@@ -496,7 +609,7 @@ class LambdaDeployer:
     def read_config_info(self):
         try:
             ini_file = 'config/config.ini'
-            config = ConfigParser.ConfigParser()
+            config = ConfigParser()
             config.read(ini_file)
             the_stuff = {}
             for section in config.sections():
@@ -531,6 +644,7 @@ class LambdaDeployer:
             self._s3_client = self._b3Sess.client('s3')
             self._cf_client = self._b3Sess.client('cloudformation', region_name=self._region)
             self._ssm_client = self._b3Sess.client('ssm', region_name=self._region)
+            self._apig_client = self._b3Sess.client('apigateway', region_name=self._region)
 
             return True
         except Exception as wtf:
